@@ -1,7 +1,7 @@
 package Archive::BagIt::Fast;
 use strict;
 use warnings;
-use Carp qw( croak );
+use Carp qw( carp croak );
 use Time::HiRes qw( time );
 use Moo;
 use IO::AIO ();
@@ -9,43 +9,49 @@ use Net::SSLeay ();
 extends "Archive::BagIt";
 
 
-our $VERSION = '0.082'; # TRIAL VERSION
+our $VERSION = '0.084'; # TRIAL VERSION
 
-# ABSTRACT: A module to use IO::AIO to get better performance
+# ABSTRACT: A module to use L<IO::AIO> to get better performance
 
 
 sub BEGIN {
     Net::SSLeay::OpenSSL_add_all_digests();
 }
 
-sub _small_digest {
+sub _XXX_digest {
+    my $digestobj = shift;
+    my $data_ref = shift;
+    my $data = ${ $data_ref };
+    Net::SSLeay::EVP_DigestUpdate($digestobj->_digest, $data);
+    my $result = Net::SSLeay::EVP_DigestFinal($digestobj->_digest);
+    Net::SSLeay::EVP_MD_CTX_destroy($digestobj->_digest);
+    delete $digestobj->{_digest};
+    my $digest = unpack('H*', $result);
+    return $digest;
+
+}
+
+sub sysread_based_digest {
     my $digestobj = shift;
     my $fh = shift;
     my $filesize = shift;
     my $data;
     sysread $fh, $data, $filesize;
-    Net::SSLeay::EVP_DigestUpdate($digestobj->_digest, $data);
-    my $result = Net::SSLeay::EVP_DigestFinal($digestobj->_digest);
-    Net::SSLeay::EVP_MD_CTX_destroy($digestobj->_digest);
-    delete $digestobj->{_digest};
-    my $digest = unpack('H*', $result);
-    return $digest;
+    return _XXX_digest( $digestobj, \$data);
 }
 
-sub _large_digest {
+sub mmap_based_digest {
     my $digestobj = shift;
     my $fh = shift;
     my $filesize = shift;
     my $data='';
     if ($filesize > 0) {
-        IO::AIO::mmap $data, $filesize, IO::AIO::PROT_READ, IO::AIO::MAP_SHARED, $fh or croak "mmap: $!";
+        if (! IO::AIO::mmap $data, $filesize, IO::AIO::PROT_READ, IO::AIO::MAP_SHARED, $fh) {
+            carp "mmap fails, fall back to sysread";
+            sysread $fh, $data, $filesize;
+        };
     }
-    Net::SSLeay::EVP_DigestUpdate($digestobj->_digest, $data);
-    my $result = Net::SSLeay::EVP_DigestFinal($digestobj->_digest);
-    Net::SSLeay::EVP_MD_CTX_destroy($digestobj->_digest);
-    delete $digestobj->{_digest};
-    my $digest = unpack('H*', $result);
-    return $digest;
+    return _XXX_digest($digestobj, \$data);
 }
 
 has 'digest_callback' => (
@@ -74,10 +80,10 @@ has 'digest_callback' => (
             my $start_time = time();
             my $digest;
             if ($filesize < $MMAP_MIN ) {
-                return _small_digest($digestobj, $fh, $filesize);
+                return sysread_based_digest($digestobj, $fh, $filesize);
             }
             elsif ( $filesize < 1500000000) {
-                return _large_digest($digestobj, $fh, $filesize);
+                return mmap_based_digest($digestobj, $fh, $filesize);
             }
             else {
                 $digest = $digestobj->get_hash_string($fh);
@@ -102,11 +108,11 @@ __END__
 
 =head1 NAME
 
-Archive::BagIt::Fast - A module to use IO::AIO to get better performance
+Archive::BagIt::Fast - A module to use L<IO::AIO> to get better performance
 
 =head1 VERSION
 
-version 0.082
+version 0.084
 
 =head1 NAME
 
@@ -124,7 +130,15 @@ typical file size, filesystem and storage systems.
 =item digest_callback()
 
 register a callback function with method to calculate and return all digests for a a list of files using a Digest-object. This method implements fast
-file access using memory mapped I/O by IO::AIO.
+file access using memory mapped I/O by L<IO::AIO>.
+
+=item mmap_based_digest($digestobj, $fh, $filesize)
+
+internal funtion which uses mmap to calculate digest. Called by C<digest_callback>
+
+=item sysread_based_digest($digestobj, $fh, $filesize)
+
+internal function which uses sysread to calculate digest. Called by C<digest_callback>
 
 =back
 
@@ -141,11 +155,11 @@ web interface at L<http://rt.cpan.org>.
 
 =head1 AUTHOR
 
-Rob Schmidt <rjeschmi@gmail.com>
+Andreas Romeyke <cpan@andreas.romeyke.de>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
+This software is copyright (c) 2021 by Rob Schmidt <rjeschmi@gmail.com>, William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
