@@ -1,10 +1,11 @@
 package Archive::BagIt::Plugin::Manifest::MD5;
+# ABSTRACT: The role to load md5 plugin (default for v0.97)
+our $VERSION = '0.084'; # TRIAL VERSION
+
 use strict;
 use warnings;
 use Moo;
 with 'Archive::BagIt::Role::Manifest';
-# ABSTRACT: The role to load md5 plugin (default for v0.97)
-our $VERSION = '0.082'; # TRIAL VERSION
 
 has '+plugin_name' => (
     is => 'ro',
@@ -44,7 +45,7 @@ Archive::BagIt::Plugin::Manifest::MD5 - The role to load md5 plugin (default for
 
 =head1 VERSION
 
-version 0.082
+version 0.084
 
 =head1 AVAILABILITY
 
@@ -59,11 +60,11 @@ web interface at L<http://rt.cpan.org>.
 
 =head1 AUTHOR
 
-Rob Schmidt <rjeschmi@gmail.com>
+Andreas Romeyke <cpan@andreas.romeyke.de>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
+This software is copyright (c) 2021 by Rob Schmidt <rjeschmi@gmail.com>, William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
